@@ -31,9 +31,22 @@ CODE_CELL = u'# <codecell>\n{}\n# </codecell>\n'
 # Matches <codecell> ... </codecell> and <markdowncell> ... </markdowncell>
 NOTEBOOK_PATTERN = r'^#[ \t]*<(?P<cell_type>code|markdown)cell>[ \t]*\n(?P<source>.*?)\n^#[ \t]*</(code|markdown)cell>'
 # Matches # %% .. # %%
-SPYDER_PATTERN = r'((#[ \t]*%%[ \t]*\n)|\A)(?P<source>.*?)(\n|\Z)((?=#[ \t]*%%[ \t]*\n)|\Z)'
+SPYDER_PATTERN = r'((#[ \t]*%%[ \t]*\n)|\A)(?P<source>.*?)(\n|\Z)((?=#[ \t]*%%[ \t]*(\n|\Z))|\Z)'
+# Matches based on multiline strings """ or '''
+SIMPLE_PATTERN = r'^["\']{3}[ \t]*\n(?P<source>.*?)\n["\']{3}[ \t]*\n'
 # To check whether there any Spyder cells in there
-SPYDER_HAS_CELLS  = r'#[ \t]*%%[ \t]*\n'
+SPYDER_HAS_CELLS  = r'#[ \t]*%%[ \t]*(\n|\Z)'
+# Matches ``` and ~~~ for code blocks embedded in Markdown
+FENCED_BLOCK_RE = re.compile(
+r'''
+(?P<fence>^(?:~{3,}|`{3,}))[ ]*                      # opening fence
+((\{(?P<attrs>[^\}\n]*)\})?|                         # (optional {attrs} or
+(\.?(?P<lang>[\w#.+-]*))?[ ]*                        # optional (.)lang
+(hl_lines=(?P<quot>"|')(?P<hl_lines>.*?)(?P=quot))?) # optional hl_lines)
+[ ]*\n                                               # newline (end of opening fence)
+(?P<code>.*?)(?<=\n)                                 # the code block
+(?P=fence)[ ]*$                                      # closing fence
+''', re.MULTILINE | re.DOTALL | re.VERBOSE)
 
 
 class JupyterNotebook(BaseExtension):
@@ -74,49 +87,27 @@ class JupyterNotebook(BaseExtension):
         return self._widget
 
     def provide_jupyter_notebook_cells(self, code=u'', cell_types=None):
-
-        cells = []
-        # Notebook type cells
-        for m in re.finditer(NOTEBOOK_PATTERN, code, re.MULTILINE | re.DOTALL):
-            if (
-                cell_types is not None and
-                m.group('cell_type') not in cell_types
-            ):
-                continue
-            cells.append({
-                'cell_type': m.group('cell_type'),
-                'source': m.group('source'),
-                'start': m.start(),
-                'end': m.end()
-            })
-        # Spyder type cells. We only search for those if there's at least one
-        # Spyder cell definition in the code, because otherwise we match the
-        # entire code.
-        if (
-            (cell_types is None or 'code' in cell_types) and
-            re.search(SPYDER_HAS_CELLS, code, re.DOTALL) is not None
-        ):
-            for m in re.finditer(
-                SPYDER_PATTERN,
-                code,
-                re.MULTILINE | re.DOTALL
-            ):
-                cells.append({
-                    'cell_type': 'code',
-                    'source': m.group('source'),
-                    'start': m.start(),
-                    'end': m.end()
-                })
-        return cells
-
-    def _import_ipynb(self):
-
-        path = QFileDialog.getOpenFileName(
-            self.main_window,
-            _(u'Open Jupyter/ IPython Notebook'),
-            filter=u'Notebooks (*.ipynb)',
-            directory=cfg.file_dialog_path
+        
+        fnc_name = '_{}_cells'.format(
+            self.extension_manager.provide('ide_current_language')
         )
+        if not hasattr(self, fnc_name):
+            return
+        return getattr(self, fnc_name)(code, cell_types)
+    
+    def provide_open_file_extension_ipynb(self):
+        
+        return self._import_ipynb, _('Import as Python script')
+    
+    def _import_ipynb(self, path=None):
+
+        if path is None:
+            path = QFileDialog.getOpenFileName(
+                self.main_window,
+                _(u'Open Jupyter/ IPython Notebook'),
+                filter=u'Notebooks (*.ipynb)',
+                directory=cfg.file_dialog_path
+            )
         if isinstance(path, tuple):
             path = path[0]
         if not path:
@@ -129,6 +120,12 @@ class JupyterNotebook(BaseExtension):
 
     def _export_ipynb(self):
 
+        if self.extension_manager.provide('ide_current_language') != 'python':
+            self.extension_manager.fire(
+                'notify',
+                message=_(u'Only Python code can be exported to a notebook')
+            )
+            return
         path = QFileDialog.getSaveFileName(
             self.main_window,
             _(u'Open Jupyter/ IPython Notebook'),
@@ -159,9 +156,9 @@ class JupyterNotebook(BaseExtension):
         py_cells = []
         for cell in nb['cells']:
             if cell['cell_type'] == 'markdown':
-                py_cells.append(MARKDOWN_CELL.format(cell['source']))
+                py_cells.append(u'"""\n{}\n"""\n'.format(cell['source']))
             elif cell['cell_type'] == 'code':
-                py_cells.append(CODE_CELL.format(cell['source']))
+                py_cells.append(cell['source'] + u'\n')
         return u'\n'.join(py_cells)
 
     def _code_to_notebook(self, code, path):
@@ -169,17 +166,117 @@ class JupyterNotebook(BaseExtension):
         import nbformat
 
         nb = nbformat.v4.new_notebook()
-        for m in re.finditer(NOTEBOOK_PATTERN, code, re.MULTILINE | re.DOTALL):
+        for pycell in self._python_cells(code):
             cell = {
-                'cell_type': m.group('cell_type'),
-                'source': m.group('source'),
+                'cell_type': pycell['cell_type'],
+                'source': pycell['source'],
                 'metadata': {}
             }
-            if m.group('cell_type') == 'code':
+            if pycell['cell_type'] == 'code':
                 cell['execution_count'] = 0
                 cell['outputs'] = []
-            elif m.group('cell_type') == 'markdown':
+            elif pycell['cell_type'] == 'markdown':
                 cell['source'] = \
                     cell['source'].lstrip(u'"""\n').rstrip(u'\n"""')
             nb['cells'].append(nbformat.from_dict(cell))
         nbformat.write(nb, path)
+
+    def _python_cells(self, code=u'', cell_types=None):
+
+        cells = []
+        # Notebook type cells
+        for m in re.finditer(NOTEBOOK_PATTERN, code, re.MULTILINE | re.DOTALL):
+            if (
+                cell_types is not None and
+                m.group('cell_type') not in cell_types
+            ):
+                continue
+            cells.append({
+                'cell_type': m.group('cell_type'),
+                'source': m.group('source'),
+                'start': m.start(),
+                'end': m.end()
+            })
+        if cells:
+            return cells
+        # Spyder type cells. We only search for those if there's at least one
+        # Spyder cell definition in the code, because otherwise we match the
+        # entire code.
+        if (
+            (cell_types is None or 'code' in cell_types) and
+            re.search(SPYDER_HAS_CELLS, code, re.DOTALL) is not None
+        ):
+            for m in re.finditer(
+                SPYDER_PATTERN,
+                code,
+                re.MULTILINE | re.DOTALL
+            ):
+                cells.append({
+                    'cell_type': 'code',
+                    'source': m.group('source'),
+                    'start': m.start(),
+                    'end': m.end()
+                })
+        if cells:
+            return cells
+        # Simple cells, separated by triple quotes
+        end_prev = 0
+        for m in re.finditer(SIMPLE_PATTERN, code, re.MULTILINE | re.DOTALL):
+            # Code cells in between two comments
+            if cell_types is None or u'code' in cell_types:
+                codecell = code[end_prev:m.start()].strip()
+                # Ignore empty codecells at the beginning
+                if codecell or end_prev:
+                    cells.append({
+                        'cell_type': 'code',
+                        'source': codecell,
+                        'start': end_prev,
+                        'end': m.start()
+                    })
+            if cell_types is None or u'markdown' in cell_types:
+                cells.append({
+                    'cell_type': 'markdown',
+                    'source': m.group('source'),
+                    'start': m.start(),
+                    'end': m.end()
+                })
+            end_prev = m.end()
+        # The last code cell that is not followed by a comment
+        if (
+            (cell_types is None or u'code' in cell_types) and
+            end_prev != len(code) - 1
+        ):
+            codecell = code[end_prev:].strip()
+            if codecell:
+                cells.append({
+                    'cell_type': 'code',
+                    'source': codecell,
+                    'start': end_prev,
+                    'end': len(code) - 1
+                })
+        return cells
+
+    def _R_cells(self, code=u'', cell_types=None):
+        
+        return self._python_cells(code, cell_types)
+
+    def _markdown_cells(self, code=u'', cell_types=None):
+        
+        cells = []
+        offset = 0
+        while True:
+            m = FENCED_BLOCK_RE.search(code)
+            if not m:
+                break
+            chunk = code[m.start():m.end()]
+            start = m.start() + offset + chunk.find(u'\n') + 1
+            end = m.start() + offset + chunk.rfind(u'\n')
+            cells.append({
+                'cell_type': 'code',
+                'source': m.group('code'),
+                'start': start,
+                'end': end 
+                })
+            offset += m.end()
+            code = code[m.end():]
+        return cells
